﻿using BPA.KitChen.StoreManagement.Application.BaseDto;
using BPA.KitChen.StoreManagement.Application.Service.ShopManage.FoodMenu.Dtos;
using BPA.KitChen.StoreManagement.Application.Service.ShopManage.Printer;
using BPA.KitChen.StoreManagement.Core.Common;
using BPA.KitChen.StoreManagement.Core.Common.Const;
using BPA.KitChen.StoreManagement.Core.Entity;
using BPA.KitChen.StoreManagement.Core.Enum;
using BPA.KitChen.StoreManagement.SqlSugar;
using Furion;
using Furion.DatabaseAccessor;
using Furion.DependencyInjection;
using Furion.DynamicApiController;
using Furion.FriendlyException;
using Furion.JsonSerialization;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Newtonsoft.Json;
using SqlSugar;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BPA.KitChen.StoreManagement.Application.Service.ShopManage.FoodMenu
{

    [ApiDescriptionSettings("店铺管理", Tag = "菜谱管理", SplitCamelCase = false)]
    public class FoodMenuServices : SupperRepository, IFoodMenuServices, IDynamicApiController, ITransient
    {
        /// <summary>
        /// 核心对象：拥有完整的SqlSugar全部功能
        /// </summary>
        private readonly SqlSugarScope db;

        private readonly IHttpContextAccessor _httpContextAccessor;
        /// <summary>
        /// construct
        /// </summary>
        /// <param name="sqlSugarRepository"></param>
        public FoodMenuServices(IHttpContextAccessor httpContextAccessor)
        {
            db = SqlSugarDb.Db;
            _httpContextAccessor = httpContextAccessor;
        }

        /// <summary>
        /// 获取门店的菜谱
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        [HttpPost("/api/FoodMenu/GetShopFoodMenuPage")]
        public async Task<PageUtil> GetShopFoodMenuPageAsync(StoreFoodMenuPageInputDto inputDto)
        {

            int total = 0;
            var data = db.Queryable<BPA_FoodMenuInfo>()
                .Where(a => a.IsDeleted == 0 && a.StoreId == inputDto.StoreId)
                .OrderBy(a => a.CreateAt, OrderByType.Desc)
                .Select(a => new
                {
                    Id = a.Id,
                    Name = a.Name,
                    Sort = a.Sort,
                })
                .ToPageList(inputDto.Current, inputDto.PageSize, ref total);

            return new PageUtil()
            {
                Data = data,
                Total = total

            };
        }



        /// <summary>
        /// 添加修改菜谱
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        [HttpPost("/api/FoodMenu/AddUpdateFoodMenu")]
        public async Task<bool> AddUpdateFoodMenuAsync(FoodMenuInfoBaseDto inputDto)
        {
            var obj1 = await db.Queryable<BPA_FoodMenuInfo>()
                .Where(x => x.Code == inputDto.Code || x.Name == inputDto.Name).FirstAsync();
            if (!string.IsNullOrWhiteSpace(inputDto.Id))
            {
                var data = await db.Queryable<BPA_FoodMenuInfo>().Where(x => x.Id == inputDto.Id).FirstAsync();


                if (obj1 != null && data.Id != obj1.Id)
                {
                    throw new Exception("名称或编码重复");
                }

                if (data != null)
                {
                    var obj = await db.Queryable<BPA_FoodMenuInfo>()
                        .Where(x => x.Code == inputDto.Code && x.Id != inputDto.Id).FirstAsync();
                    if (obj != null)
                    {

                        return false;
                    }
                    data.Name = inputDto.Name;
                    data.Code = inputDto.Code;
                    data.Sort = inputDto.Sort;
                    data.Status = (CommonStatus)inputDto.Status;
                    var res = await db.Updateable<BPA_FoodMenuInfo>(data).CallEntityMethod(m => m.Modify()).ExecuteCommandAsync();
                    return res > 0;
                }
                else
                {
                    var obj = await db.Queryable<BPA_FoodMenuInfo>()
                        .Where(x => x.Code == inputDto.Code).FirstAsync();
                    if (obj != null)
                    {
                        return false;
                    }
                    var res = await db.Insertable(new BPA_FoodMenuInfo
                    {
                        StoreId = inputDto.StoreId,
                        Name = inputDto.Name,
                        Code = inputDto.Code,
                        Sort = inputDto.Sort,
                        Id = Guid.NewGuid().ToString(),
                        Status = (CommonStatus)inputDto.Status

                    }).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                    return res > 0;
                }
            }
            else
            {

                if (obj1 != null && string.IsNullOrEmpty(inputDto.Id))
                {
                    throw new Exception("名称或编码重复");
                }
                var obj = await db.Queryable<BPA_FoodMenuInfo>()
                    .Where(x => x.Code == inputDto.Code).FirstAsync();
                if (obj != null)
                {
                    return false;
                }

                var res = await db.Insertable(new BPA_FoodMenuInfo
                {
                    StoreId = inputDto.StoreId,
                    Name = inputDto.Name,
                    Code = inputDto.Code,
                    Sort = inputDto.Sort,
                    Id = Guid.NewGuid().ToString(),
                    Status = (CommonStatus)inputDto.Status

                }).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                return res > 0;
            }

        }


        /// <summary>
        /// 获取菜品列表
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        [HttpPost("/api/FoodMenu/GetGoodsByMenuIdPage")]
        public async Task<PageUtil> GetGoodsByMenuIdPage(FoodMenuQueryInputDto inputDto)
        {
            var headers = _httpContextAccessor.HttpContext.Request.Headers;
            var authorization = headers["Authorization"];

            Dictionary<string, string> dic = new Dictionary<string, string>();
            dic.Add("Authorization", authorization);
            //var url1 = App.Configuration["SAAS_Manage"] + "/api/goods/getgoodslist";
            //var json1 = HttpHelper.HttpGet(url1, "", dic, "");

            var url = App.Configuration["SAAS_Manage"] + "/api/goods/page";
            var json2 = HttpHelper.PostData(url, JsonConvert.SerializeObject(new
            {
                current = inputDto.Current,
                pageSize = inputDto.PageSize,
                name=inputDto.Name,

            }), Encoding.UTF8, "application/json", dic);

            var data = JsonConvert.DeserializeObject<BaseResultDto>(json2);


            var result = new List<GoodsInfoCheck>();
            int total = 0;

            var foodMenuList = await db.Queryable<BPA_FoodMenu>().Where(c => c.FoodMenuId == inputDto.FoodMenuId).ToListAsync();
            if (data?.data != null)
            {
                foreach (var item in data?.data?.data)
                {
                    var check = foodMenuList.Any(x => x.FoodMenuId == item.id);

                    result.Add(new GoodsInfoCheck()
                    {
                        Id = item.id,
                        Name = item.name,
                        Check = check
                    });
                }
            }

            return new PageUtil()
            {
                Data = result,
                Total = data?.data?.total??0,
            };
        }

        /// <summary>
        /// 获取菜品列表
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        [HttpPost("/api/FoodMenu/GetFoodMenuGoodsPage_V2")]
        public async Task<PageUtil> GetFoodMenuGoodsPageAsync_V2(FoodMenuQueryInputDto inputDto)
        {
            int total = 0;
            var data = db.Queryable<BPA_FoodMenu>()
                .Where((a) => a.IsDeleted == 0 && a.FoodMenuId == inputDto.FoodMenuId)
                .OrderBy((a) => a.CreateAt, OrderByType.Desc)
                .Select((a) => new
                {
                    Id = a.Id,
                    Goodid = a.GoodsId,
                    Name = a.GoodsName
                })
                .ToPageList(inputDto.Current, inputDto.PageSize, ref total);

            return new PageUtil()
            {
                Data = data,
                Total = total

            };
        }

        /// <summary>
        /// 编辑菜谱商品
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        /// <exception cref="Exception"></exception>
        [HttpPost("/api/FoodMenu/EditFoodMenuGoods")]
        public async Task<bool> EditFoodMenuGoods(FoodMenuBaseDto inputDto)
        {
            try
            {
                var headers = _httpContextAccessor.HttpContext.Request.Headers;
                var authorization = headers["Authorization"];

                Dictionary<string, string> dic = new Dictionary<string, string>();
                dic.Add("Authorization", authorization);
                var url1 = App.Configuration["SAAS_Manage"] + "/api/goods/getgoodslist";
                var json1 = HttpHelper.HttpGet(url1, "", dic, "");

                var data2 = JsonConvert.DeserializeObject<BaseResultDto2>(json1);

                db.Ado.BeginTran();
                //删除设置的菜谱
                await db.Deleteable<BPA_FoodMenu>()
                    .Where(x => inputDto.FoodMenuId.Contains(x.FoodMenuId)).ExecuteCommandAsync();
                //菜谱添加商品
                var data = inputDto.GoodsIds.Select(x => new BPA_FoodMenu()
                {
                    FoodMenuId = inputDto.FoodMenuId,
                    GoodsName = data2?.data?.First(c => c.id == x.GoodsId)?.name,
                    GoodsId = x.GoodsId,
                }).ToList();
                if (data.Count > 0)
                {
                    var res = await db.Insertable(data).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                }
                //查询出店铺关联的当前菜谱菜品

                var storeFoodMenu = await db.Queryable<BPA_StoreFoodMenu>()
                    .Where(x => x.FoodMenuId == inputDto.FoodMenuId).ToListAsync();

                var res1 = db.Deleteable<BPA_StoreFoodMenu>().Where(x => x.FoodMenuId == inputDto.FoodMenuId).ExecuteCommand();

                foreach (var item in storeFoodMenu.GroupBy(x => x.StoreId))
                {
                    var thisData = storeFoodMenu.Where(x => x.StoreId == item.First().StoreId).ToList();
                    foreach (var item2 in inputDto.GoodsIds)
                    {
                        var tihsAdd = thisData.Where(x => x.FoodId == item2.GoodsId).FirstOrDefault();

                        await db.Insertable(new BPA_StoreFoodMenu()
                        {
                            CookingWorkId = tihsAdd?.CookingWorkId ?? "",
                            Discount2Price = tihsAdd?.Discount2Price ?? 0,
                            DiscountPrice = tihsAdd?.DiscountPrice ?? 0,
                            FoodId = tihsAdd?.FoodId ?? item2.GoodsId,
                            FoodMenuId = inputDto.FoodMenuId,
                            FoodName = item?.First()?.FoodName,
                            Price = tihsAdd?.Price ?? 0,
                            StopSales = tihsAdd?.StopSales ?? 0,
                            StoreId = item.First().StoreId,
                            Vip2Price = tihsAdd?.Vip2Price ?? 0,
                            VipPrice = tihsAdd?.VipPrice ?? 0,
                        }).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();

                    }

                }

                await StoreAddFoodMenuGoodsNewAsync(new StoreFoodMenuNewDto()
                {
                    StoreId = inputDto.StoreId,
                    FoodMenuIds = new List<string>
                    {
                        inputDto.FoodMenuId
                    }
                });

                db.Ado.CommitTran();
            }
            catch (Exception ex)
            {
                db.Ado.RollbackTran();
                throw new Exception("编辑错误");
            }

            return true;

        }

        [HttpPost("/api/FoodMenu/GetStoreGoods")]
        public async Task<PageUtil> GetStoreGoodsAsync(StoreGoodsQueryInputDto inputDto)
        {
            var total = new RefAsync<int>();
            var data = await db
                .Queryable<BPA_StoreFoodMenu>()
                .Where((s) => s.IsDeleted == 0)
                .Where((s) => s.StoreId == inputDto.StoreId)
                .WhereIF(!string.IsNullOrEmpty(inputDto.FoodMenuId), (s) => s.FoodMenuId == inputDto.FoodMenuId)
                .WhereIF(!string.IsNullOrWhiteSpace(inputDto.GoodsName), (s) => s.FoodName.Contains(inputDto.GoodsName))
                .WhereIF(inputDto.IsDevice != null, (s) => s.IsDevice == inputDto.IsDevice)
                .WhereIF(inputDto.Status != null, (s) => s.Status == inputDto.Status)
                .WhereIF(inputDto.StopSales != null, (s) => s.StopSales == inputDto.StopSales)
                .OrderBy((s) => s.IsDevice, OrderByType.Asc)
                .Select((s) => new StoreGoodsBaseDto()
                {
                    Id = s.Id.SelectAll(),
                    GoodsId = s.FoodId,
                    Goods_Name = s.FoodName
                })
                .ToPageListAsync(inputDto.Current, inputDto.PageSize, total);
            return new PageUtil()
            {
                Data = data,
                Total = total
            };
        }


        /// <summary>
        /// 店铺添加菜谱new
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        [HttpPost("/api/FoodMenu/StoreAddFoodMenuNew")]
        public async Task<bool> StoreAddFoodMenuGoodsNewAsync(StoreFoodMenuNewDto inputDto)
        {
            var data = db.Queryable<BPA_FoodMenu>()
                .Where((a) => a.IsDeleted == 0 && inputDto.FoodMenuIds.Contains(a.FoodMenuId))
                .OrderBy((a) => a.CreateAt, OrderByType.Desc)
                .Select((a) => new
                {
                    Id = a.Id,
                    goodsId = a.Id,
                    goodsName=a.GoodsName,
                    Price = 0,
                    vipPrice = 0,
                    a.FoodMenuId
                }).ToList();



            //删除门店下所有菜谱商品
            db.Deleteable<BPA_StoreFoodMenu>()
                .Where(x => x.StoreId == inputDto.StoreId&& inputDto.FoodMenuIds.Contains(x.FoodMenuId))
                .ExecuteCommand();

            List<BPA_StoreFoodMenu> bPA_StoreFoods = new List<BPA_StoreFoodMenu>();

            foreach (var item in data)
            {
                BPA_StoreFoodMenu bPA_StoreFood = new BPA_StoreFoodMenu();
                bPA_StoreFood.FoodMenuId = item.FoodMenuId;
                bPA_StoreFood.StoreId = inputDto.StoreId;
                bPA_StoreFood.FoodId = item.goodsId;
                bPA_StoreFood.FoodName = item.goodsName;
                bPA_StoreFood.Price = item.Price;
                bPA_StoreFood.VipPrice = item.vipPrice;
                bPA_StoreFood.IsDevice = 0;
                bPA_StoreFoods.Add(bPA_StoreFood);
            }


            if (bPA_StoreFoods.Count > 0)
            {
                return db.Insertable(bPA_StoreFoods).CallEntityMethod(it => it.Create()).ExecuteCommand() > 0;
            }
            else
            {
                return false;
            }
        }


        /// <summary>
        /// 修改
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        [HttpPut("/api/FoodMenu/UpdateStoreGood")]
        public async Task<bool> UpdateStoreGoodAsync(StoreGoodsBaseDto inputDto)
        {
            if (!string.IsNullOrWhiteSpace(inputDto.Id))
            {
                var data = await db.Queryable<BPA_StoreFoodMenu>().Where(x => x.Id == inputDto.Id).FirstAsync();

                if (data != null)
                {
                    if (inputDto.Price != null && inputDto.Price > 0)
                    {
                        data.Price = inputDto.Price;

                    }
                    if (inputDto.VipPrice != null && inputDto.VipPrice > 0)
                    {
                        data.VipPrice = inputDto.VipPrice;
                    }
                    data.Status = inputDto.Status;
                    data.IsDevice = inputDto.IsDevice;
                    var res = await db.Updateable<BPA_StoreFoodMenu>(data)
                        .UpdateColumns(x => new
                        {
                            x.Price,
                            x.Status,
                            x.VipPrice,
                            x.IsDevice,
                        })
                        .Where(x => x.Id == inputDto.Id).ExecuteCommandAsync();
                    return res > 0;
                }
                return true;
            }
            return false;
        }


        /// <summary>
        /// 估清
        /// </summary>
        /// <param name="stopGoods"></param>
        /// <returns></returns>
        [HttpPost("/api/FoodMenu/SetStopGoods")]
        public bool SetStopGoods(List<StopGoodsDto> stopGoods)
        {
            if (stopGoods.Count() <= 0)
            {
                return false;
            }
            string[] goodIds = stopGoods.Select(a => a.GoodsId).ToArray();
            string storeId = stopGoods.FirstOrDefault().StoreId;
            List<BPA_StoreFoodMenu> storefoods = db.Queryable<BPA_StoreFoodMenu>()
                .Filter(null, true)
                .Where(a => a.StoreId == storeId && goodIds.Contains(a.FoodId)).ToList();
            if (storefoods.Count() <= 0)
            {
                return false;
            }
            for (int i = 0; i < storefoods.Count(); i++)
            {
                string goodsId = storefoods[i].FoodId;
                var stopgoods = stopGoods.Where(a => a.GoodsId == goodsId).FirstOrDefault();
                storefoods[i].StopSales = stopgoods.StopSales;
                if (stopgoods.StopSales == 1)
                {
                    if (stopgoods.StopTime.HasValue)
                    {
                        storefoods[i].StopSalesTime = stopgoods.StopTime.Value;
                    }
                    else
                    {
                        storefoods[i].StopSalesTime = Convert.ToDateTime(DateTime.Now.AddDays(1).ToString("D").ToString()).AddSeconds(-1);
                    }
                }
            }

            db.Updateable(storefoods).CallEntityMethod(m => m.Modify()).ExecuteCommand();

            return true;
        }


        /// <summary>
        /// 商品绑定设备
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        [HttpPost("/api/FoodMenu/AddGoodDeviceAsync")]
        public async Task<bool> AddGoodDeviceAsync(GoodsDevicesInputDto inputDto)
        {
            var GroupId = App.User.FindFirst(ClaimConst.GroupId)?.Value;
            var DeviceFoodList = db.Queryable<BPA_DeviceFood>().Where(a => a.FoodId == inputDto.Foods).ToList();
            var foods = db.Queryable<BPA_StoreFoodMenu>().Where(a => a.FoodMenuId == inputDto.FoodMenuId && a.FoodId == inputDto.Foods).First();
            if (foods == null) throw Oops.Oh("没有找到菜谱中对应的商品");
            db.Deleteable(DeviceFoodList).ExecuteCommand();
            List<BPA_DeviceFood> bPA_DeviceFoods = new List<BPA_DeviceFood>();
            if (inputDto.DeviceIds.Count > 0)
            {
                inputDto.DeviceIds.ForEach(x =>
                {
                    BPA_DeviceFood DeviceFood = new BPA_DeviceFood();
                    DeviceFood.DeviceId = x.DeviceId;
                    DeviceFood.FoodId = inputDto.Foods;
                    bPA_DeviceFoods.Add(DeviceFood);
                });
                if (bPA_DeviceFoods.Count > 0)
                    await db.Insertable(bPA_DeviceFoods).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                foods.IsDevice = inputDto.IsDevice ?? 1;
            }
            else
            {
                foods.IsDevice = inputDto.IsDevice ?? 0;
            }
            var res1 = db.Updateable(foods).CallEntityMethod(m => m.Modify()).ExecuteCommand();
            return res1 > 0;
        }


        /// <summary>
        /// 添加加购商品
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("/api/FoodMenu/PlusAddGoods")]
        public async Task<bool> PlusAddGoods(PlusAddGoodsDto dto)
        {
            List<BPA_AddGoods> bPA_AddGoods = new List<BPA_AddGoods>();
            //校验是否添加过
            var headers = _httpContextAccessor.HttpContext.Request.Headers;
            var authorization = headers["Authorization"];

            Dictionary<string, string> dic = new Dictionary<string, string>();
            dic.Add("Authorization", authorization);
            var url1 = App.Configuration["SAAS_Manage"] + "/api/goods/getgoodslist";
            var json1 = HttpHelper.HttpGet(url1, "", dic, "");

            var data2 = JsonConvert.DeserializeObject<BaseResultDto2>(json1);

            var goods=data2.data?.Where(x=>x.id==dto.AddGoodsId)?.First();


            bPA_AddGoods = db.Queryable<BPA_AddGoods>().Where(a => a.GoodsId == dto.GoodsId && a.AddGoodsId == dto.AddGoodsId && a.StoreId == dto.StoreId).ToList();
            if (bPA_AddGoods.Count > 0)
            {
                BPA_AddGoods bPA_AddGood = bPA_AddGoods.First();
                bPA_AddGood.GoodsId = dto.GoodsId;
                bPA_AddGood.AddGoodsId = dto.AddGoodsId;
                bPA_AddGood.StoreId = dto.StoreId;
                bPA_AddGood.Price = dto.Price;
                bPA_AddGood.Status = CommonStatus.ENABLE;
                bPA_AddGood.IsDeleted = 0;
                bPA_AddGood.GoodsName = goods?.name;
                var res = db.Updateable(bPA_AddGood).CallEntityMethod(m => m.Modify()).ExecuteCommand();
                return res > 0;
            }
            else
            {
                BPA_AddGoods bPA_AddGood = new BPA_AddGoods();
                bPA_AddGood.GoodsId = dto.GoodsId;
                bPA_AddGood.AddGoodsId = dto.AddGoodsId;
                bPA_AddGood.StoreId = dto.StoreId;
                bPA_AddGood.Price = dto.Price;
                bPA_AddGood.GoodsName = goods?.name; 
                var res = db.Insertable(bPA_AddGood).CallEntityMethod(m => m.Create()).ExecuteCommand();
                return res > 0;
            }


        }

        /// <summary>
        /// 加购商品
        /// </summary>
        /// <param name="PageIndex"></param>
        /// <param name="PageSize"></param>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("/api/FoodMenu/GetAddGoodsPageAsync")]
        public PageUtil GetAddGoodsPageAsync(OrtherGoodsQueryDto dto)
        {
            int total = new RefAsync<int>();
            var res = db.Queryable<BPA_AddGoods>()
                .Where((a) => a.IsDeleted == 0 )
                .Where((a) => a.GoodsId == dto.GoodsId && a.StoreId == dto.StoreId)
                .Select((a) => new
                {
                    Id = a.Id,
                    Name = a.GoodsName,
                    a.Price,
                    a.Status
                })
                .ToPageList(dto.Current, dto.PageSize, ref total);
            PageUtil util = new PageUtil()
            {
                Total = total,
                Data = res
            };
            return util;
        }

        /// <summary>
        /// 删除加购商品
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("/api/FoodMenu/BatchDelPlusAddGoodsAsync")]
        public async Task<bool> BatchDelPlusAddGoodsAsync(string[] Ids)
        {

            try
            {
                db.BeginTran();
                var res = db.Deleteable<BPA_AddGoods>().In(Ids).ExecuteCommand();
               
                db.CommitTran();
                return res > 0;
            }
            catch (Exception ex)
            {
                db.RollbackTran();
                //BPALog.WriteLog(msg + ":" + ex.Message, LogEnum.Error, null, ex);
             return false;
            }
        }


        /// <summary>
        /// 根据商品id查询商品绑定的所有设备
        /// </summary>
        /// <param name="foodId"></param>
        /// <returns></returns>
        [HttpGet("/api/FoodMenu/GetGoodDeviceAsync")]
        public async Task<List<string>> GetGoodDeviceAsync(string foodId, string storeId)
        {
            var GroupId = App.User.FindFirst(ClaimConst.GroupId)?.Value;
            var DeviceFoodList = db.Queryable<BPA_DeviceFood>().Where((a) => a.GroupId == GroupId && a.FoodId == foodId).Select(a => a.DeviceId).ToList();
            return DeviceFoodList;
        }

    }
}
